@@ -34,10 +34,6 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import absolute_import
 
-from future import standard_library
-standard_library.install_aliases()
-from builtins import range
-from builtins import object
 import os
 import sys
 import datetime
