@@ -14,17 +14,17 @@
 """
 
 from __future__ import print_function
-from builtins import str
-from builtins import zip
-from builtins import next
-from builtins import object
 import argparse
 import functools
 import inspect
 import itertools
+from pyng.exc import describe
 import re
 import sys
 
+class Error(Exception):
+    pass
+
 def print_non_None(value):
     """
     Default formatter for values returned by functions decorated with Commands.
@@ -161,6 +161,12 @@ class Commands(object):
         from the first paragraph of the decorated function's docstring, up to
         the first blank line.
 
+        Special case: if exactly one function is decorated, the returned
+        ArgumentParser supports that function directly, rather than accepting
+        a subcommand with the name of that function. This is useful for a
+        trivial script module that nonetheless should present a plausible
+        command-line interface for its one (main()?) function.
+
         If score is overridden with (e.g.) score='-', then underscores in the
         function name produce hyphens in the subcommand name. Thus:
 
@@ -212,8 +218,15 @@ class Commands(object):
         such an argument for use by Python callers, but the ArgumentParser won't
         be affected by that argument.
         """
-        parser = ArgumentParser(description=self.help)
-        subparsers = parser.add_subparsers(title="subcommands")
+        if len(self.commands) == 1:
+            # For exactly one decorated function, return a single top-level
+            # ArgumentParser for that function.
+            parser_holder = LazyArgumentParser(self.help)
+        else:
+            # The expected case is multiple decorated functions, each getting
+            # a subcommand of its own.
+            parser_holder = ArgumentParserWithSubparsers(self.help)
+
         for func, formatter in self.commands:
             # convert function name according to decamel and score
             name = func.__name__.replace('_', score)
@@ -240,8 +253,8 @@ class Commands(object):
                 # and rejoin to a single string
                 help = '\n'.join(help)
 
-            # Add a subparser for this subcommand.
-            subparser = subparsers.add_parser(name, help=help)
+            # Add a (sub)parser for this (sub)command.
+            subparser = parser_holder.get_subparser(name, help=help)
             # and set that parser's func attribute to the function
             # also store the desired formatter
             subparser.set_defaults(func=func, formatter=formatter)
@@ -264,7 +277,7 @@ class Commands(object):
             # Add a switch for each optional argument.
             for arg, value in zip(optional, args.defaults or ()):
                 # the switch is often -- but not always -- the arg name
-                switch = '--' + arg
+                switch = '--' + arg.replace('_', score)
                 default = "(default {})".format(value)
                 # Check the default value.
                 # For True and False, use 'is' --
@@ -309,7 +322,8 @@ class Commands(object):
                 subparser.add_argument(args.varargs, nargs="*",
                                        help=self.get_arg_help(doc, args.varargs))
 
-        return parser
+        # Return the top-level ArgumentParser, whether or not we set it lazily.
+        return parser_holder.get_parser()
 
     # https://codereview.stackexchange.com/a/185973
     # Recognize a capital letter followed by lowercase letters so that (e.g.)
@@ -362,6 +376,51 @@ class Commands(object):
         # didn't find 'arg:'
         return None
 
+class LazyArgumentParser(object):
+    """
+    Instantiate one top-level ArgumentParser on demand. get_subparser() must
+    be called exactly once before get_parser(). This is for use when exactly
+    one function is decorated with a Commands instance.
+    """
+    def __init__(self, help):
+        self.help = help
+        self.parser = None
+
+    def get_subparser(self, name, help):
+        if self.parser is not None:
+            raise Error('LazyArgumentParser.get_subparser() must be called exactly once')
+        # Our caller has only just looked at this one decorated function, so
+        # this is the first time we discover its help content. If Commands was
+        # constructed with a help string, use that, else use the function's help.
+        self.parser = ArgumentParser(description=self.help or help)
+        # Let caller add argument definitions directly to this top-level
+        # parser, instead of subcommand parsers.
+        return self.parser
+
+    def get_parser(self):
+        if self.parser is None:
+            raise Error('LazyArgumentParser.get_subparser() must be called exactly once '
+                        'before get_parser()')
+        return self.parser
+
+class ArgumentParserWithSubparsers(object):
+    """
+    Immediately instantiate a top-level ArgumentParser, and make
+    get_subparser() deliver a new subcommand parser every call. This is for
+    use when multiple functions are decorated with a Commands instance.
+    """
+    def __init__(self, help):
+        self.parser = ArgumentParser(description=help)
+        self.subparsers = self.parser.add_subparsers(title="subcommands")
+
+    def get_subparser(self, name, help):
+        # Create a new subcommand parser with specified name and help. Let
+        # caller add argument definitions to the subcommand parser.
+        return self.subparsers.add_parser(name, description=help)
+
+    def get_parser(self):
+        return self.parser
+
 class ArgumentParser(argparse.ArgumentParser):
     """
     This subclass of argparse.ArgumentParser overrides parse_args() to return
@@ -410,4 +469,4 @@ def run(self, exceptions=_NeverRaised):
     try:
         return formatter(func(**kwds))
     except exceptions as err:
-        sys.exit(str(err))
+        sys.exit(describe(err))
