@@ -8,7 +8,6 @@
 Copyright (c) 2011, Nat Goodspeed
 """
 
-from builtins import object
 import cgitb
 import sys
 from contextlib import contextmanager
@@ -18,6 +17,9 @@ import itertools
 # default parameter distinct from None so None can validly be passed
 _OMITTED = object()
 
+class Error(Exception):
+    pass
+
 # TODO:
 # - fallback(func, *args, **kwds, _exc=Exception, _default=None) function
 #   returns func(*args, **kwds) unless exception _exc, in which case it
@@ -242,6 +244,108 @@ class retry(object):
                               *args, **kwds)
         return wrapper
 
+class Retry(object):
+    """
+    Retry presents yet another retry paradigm. First instantiate a Retry
+    object, binding the callable and its arguments, then iterate over fails().
+    The body of the 'for' loop is ... whatever you want to do between retries:
+    log a message, sleep(), whatever. Or count failures and break the loop.
+
+    When you drop out of the 'for' loop, the return value of the call is in
+    the Retry instance's 'result' property. If the last call still raised an
+    exception, accessing 'result' re-raises the same exception.
+
+    If the callable succeeds on the first call, of course the loop will
+    iterate 0 times, and 'result' will contain its return value.
+
+    Iterate over zip(range(n), fails()) to constrain the number of retries.
+
+    If you access 'result' without calling fails(), Retry directly calls the
+    bound callable and either returns its result or raises the exception -- no
+    retries. In other words, unless you call fails():
+
+    Retry(func, *args, **kwds).result
+    is equivalent to
+    functools.partial(func, *args, **kwds)()
+    """
+    def __init__(self, func, *args, **kwds):
+        """
+        Bind a callable and its arguments to call, potentially multiple times,
+        as long as it raises (specified) exception(s).
+
+        func: any Python callable you (might) need to retry calling
+        args: positional arguments to pass to func
+        kwds: keyword arguments to pass to func
+        """
+        self.callable = functools.wraps(func)(functools.partial(func, *args, **kwds))
+        # handle the case when 'result' is accessed without ever calling
+        # 'fails()'
+        self._result = self.callable
+
+    def fails(self, exc=Exception):
+        """
+        Each time you request an item from the fails() generator, it retries
+        a call to the callable bound in the constructor. The peculiar thing
+        about fails() is that if the bound callable succeeds, iteration
+        terminates. If it fails (raises 'exc'), fails() yields the caught
+        exception object.
+
+        exc: an Exception subclass (default Exception), or a tuple of such
+             Exception subclasses, that fails() should catch and yield
+
+        If the bound callable raises an exception NOT caught by 'except exc',
+        the exception simply propagates. In other words, the 'exc' parameter
+        lets the caller distinguish exceptions that can reasonably be retried
+        from those that must fail right away.
+
+        Usage:
+
+        retry = Retry(myfunc, 'abc', special=True)
+        for i, exc in zip(range(3), retry.fails(WhoopsError)):
+            print(f'call {i+1} failed, sleeping before retry')
+            time.sleep(1)
+        # Here either myfunc() returned a result, or it raised WhoopsError 3
+        # successive times (per the range() call above). If it returned a
+        # result, the following access will obtain it. If it raised
+        # WhoopsError every time, this statement will re-raise WhoopsError.
+        value = retry.result
+
+        Neither __init__() nor fails() supports any equivalent to our
+        retry_func(when=) parameter because it's unnecessary. retry_func()
+        calls when(exc) to discover whether it should continue looping or
+        propagate the exception to its caller. With Retry, your loop body can
+        directly examine the yielded exception object 'exc' and, if it should
+        fail instead of retrying, 'raise exc'.
+        """
+        # keep on retrying as long as caller is willing to resume:
+        # constraining the iteration is up to caller
+        while True:
+            try:
+                result = self.callable()
+            except exc as x:
+                # if caller chooses not to resume, result should re-raise
+                self._result = functools.partial(self._raise, x)
+                yield x
+            else:
+                # okay, accessing result attribute should return this
+                self._result = lambda: result
+                # once callable() succeeds, we're done
+                break
+        # once we've succeeded, calling fails() again is a usage error --
+        # point it out to the user
+        self.callable = functools.partial(
+            self._raise,
+            Error('Retry callable succeeded, fails() is done'))
+
+    @property
+    def result(self):
+        # result property is syntactic sugar for calling _result()
+        return self._result()
+
+    @staticmethod
+    def _raise(exc):
+        raise exc
+
 def describe(exc):
     """
     Return a string 'ClassName: string message', given an 'exc' of type
@@ -290,7 +394,7 @@ def rexchain(exc):
     if exc is not None:
         yield exc
 
-def caused_by(exc, classes):
+def caused_by(exc, classes, func=lambda err: err):
     """
     Given an exception instance 'exc', return which of the underlying causes
     of 'exc' is an instance of any of the tuple 'classes' (which may be passed
@@ -304,12 +408,41 @@ def caused_by(exc, classes):
 
     Python 3's OSError subclasses FileNotFoundError, FileExistsError et al.
     are particularly useful here.
+
+    Optional argument 'func' must be a callable accepting the exception
+    instance that would normally be returned. In this case caused_by() returns
+    whatever is returned by func(). If no underlying exception matches
+    'classes', func() is not called and caused_by() returns None as usual.
+
+    For example, if handling an exception from an API wrapped around the
+    Requests package, you could check specifically for 'Not Found' using code
+    such as:
+
+    try:
+        API_call(...)
+    except API_error as err:
+        # HTTPError.response might be None
+        if caused_by(err, requests.HTTPError,
+                     lambda ex: ex.response and ex.response.status_code == 404):
+            # handle not-found case
+            # ...
+
+    This is equivalent to:
+
+    try:
+        API_call(...)
+    except API_error as err:
+        ex = caused_by(err, requests.HTTPError)
+        # HTTPError.response might be None
+        if ex and ex.response and ex.response.status_code == 404):
+            # handle not-found case
+            # ...
     """
     try:
         # Loop over exchain(exc), checking each underlying exception in turn
         # to see if it's an instance of any of the passed classes. Stop and
         # return the first one we find.
-        return next(x for x in exchain(exc) if isinstance(x, classes))
+        return func(next(x for x in exchain(exc) if isinstance(x, classes)))
     except StopIteration:
         # No exception in exchain(exc) is an instance of any of 'classes'.
         return None
@@ -683,21 +816,15 @@ class cgitb_Hook(exceptlink):
         exceptlink.
         """
         system = kwds.pop('system', False)
-        self.hook = cgitb.Hook(*args, **kwds)
+        self.handle = cgitb.Hook(*args, **kwds)
         # Link this instance onto sys.excepthook AFTER preparing self.hook.
         super(cgitb_Hook, self).__init__(system)
 
-    def handle(self, type, value, traceback):
-        self.hook(type, value, traceback)
-
 class generic_exceptlink(exceptlink):
     """
     generic_exceptlink allows you to chain an arbitrary function(type, value,
     traceback) into the exceptlink chain.
     """
     def __init__(self, function, system=False):
-        self.function = function
+        self.handle = function
         super(generic_exceptlink, self).__init__(system)
-
-    def handle(self, type, value, traceback):
-        self.function(type, value, traceback)
