@@ -13,9 +13,9 @@ def filter(iterable):
 Such generators can readily be chained to construct arbitrary pipelines.
 
 The introduction of reverse data flow ('yield' returning a result) permits
-chains of output generators as well. Which processing belongs in an input
-chain and which in an output chain? The answer depends on where you put the
-driving loop. For example:
+chains of output generators ('push' generators) as well. Which processing
+belongs in an input chain and which in an output chain? The answer depends on
+where you put the driving loop. For example:
 
     from pyng.genio import filesink, filesource
     filesink(outname, filter(filesource(inname)))
@@ -57,6 +57,7 @@ import atexit
 from collections import namedtuple
 from contextlib import contextmanager
 import datetime
+from functools import wraps
 import json
 import logging
 from pyng.exc import exceptlink, describe
@@ -65,6 +66,46 @@ import syslog
 class OutError(Exception):
     pass
 
+def push_generator(func):
+    """
+    Decorate a push generator such that calling it returns a handle on which
+    you can call send(), throw(), close() et al.
+
+    @push_generator
+    def file_writer(filename):
+        with open(filename, 'w') as outf:
+            while True:
+                outf.write(yield)
+
+    Usage:
+
+    with closing(file_writer('somefile.txt')) as writer:
+        writer.send('some text')
+        # ...
+    """
+    @wraps(func)
+    def wrapper(*args, **kwds):
+        return pushiter(func(*args, **kwds))
+    return wrapper
+
+def pushiter(generator_call):
+    """
+    Return an iterator into a generator coded to accept values via 'yield': a
+    push generator. Typical usage of such an iterator would be something like:
+
+    handle = iter(generator_call(args...))
+    next(handle)                # MUST run it up to first yield!
+    handle.send(...)            # now can start pushing values
+
+    pushiter() simply encapsulates that required first next() call.
+    If your generator actually engages in bidirectional handshakes, and the
+    first yield produces a meaningful value, of course use iter() and next()
+    directly.
+    """
+    handle = iter(generator_call)
+    next(handle)                # run the generator to the first yield
+    return handle
+
 class ofilter(object):
     """
     The output filter API is primarily write() and (optionally) flush(). But
@@ -194,8 +235,6 @@ class buffer_lines(ofilter):
         """
         self.eol = kwds.pop("eol", '')
         self.loopgen = self.loop(self._maybe_adapt(sink, **kwds))
-        # Run self.loop() up to the first yield. Discard the None.
-        next(self.loopgen)
 
     def write(self, item):
         # cause loop()'s (yield) to return the passed item
@@ -207,6 +246,11 @@ class buffer_lines(ofilter):
         # but then also flush sink
         self.loopgen.throw(_Flush)
 
+    def close(self):
+        self.flush()
+        self.loopgen.close()
+
+    @push_generator
     def loop(self, sink):
         # start with an empty string so we always have a buffer[-1]
         buffer = [""]
@@ -252,6 +296,27 @@ class buffer_lines(ofilter):
 
             # Loop back to accumulate more fragments in buffer.
 
+class overtype(ofilter):
+    """
+    buffer_lines handles the case when you want a sink (e.g. a logger) to
+    process complete lines from the source. overtype handles the further case
+    when lines from the source include '\r' characters intended to replace the
+    line so far with new content, e.g. for progress output on a console.
+    That's great for a watching user, not so great for a log file after the
+    fact: you end up with lots of nearly-identical progress lines that detract
+    from readability of the log.
+
+    Expected usage would be as a sink for buffer_lines, e.g.:
+
+    outfile = buffer_lines(overtype(adapter(logger.info)))
+    """
+    def write(self, line):
+        # If, as we expect, each write() call passes a complete line
+        # (formerly) ending in newline, all we need do is split on '\r'
+        # characters and take the last chunk. In fact we need only rsplit()
+        # once.
+        super(overtype, self).write(line.rsplit('\r', 1)[-1])
+
 class _syslogger(ofilter):
     """
     An instance of this class is a 'file-like object' whose write() method
