@@ -10,9 +10,11 @@ Copyright (c) 2011, Nat Goodspeed
 
 from __future__ import with_statement   # for Python 2.5
 
+import copy
+from functools import partial
 import sys
 import unittest
-import exc
+from pyng import exc
 
 class FooError(Exception):
     pass
@@ -32,7 +34,7 @@ class TestReraise(unittest.TestCase):
         try:                            # this is outer code
             try:
                 raiser(FooError("foo"))
-            except FooError, err:
+            except FooError as err:
                 # This is cleanup code
                 try:
                     x = int("7a")
@@ -41,10 +43,12 @@ class TestReraise(unittest.TestCase):
                     x = 7
                 # except now we've reset the global exception info...
                 raise
-        except FooError, err:
+        except FooError as err:
             # This we do NOT expect, at least not in Python 2.x.
-            self.assert_(False)
-        except ValueError, err:
+            # Turns out that Python 3 handles this case correctly, meaning
+            # that reraise() isn't necessary in Python 3.
+            self.assert_(sys.version_info[0] >= 3)
+        except ValueError as err:
             self.assert_(True)
         else:
             # There better be an exception of SOME kind!
@@ -55,7 +59,7 @@ class TestReraise(unittest.TestCase):
         try:                            # this is outer code
             try:
                 raiser(FooError("foo"))
-            except FooError, err:
+            except FooError as err:
                 with exc.reraise():
                     # This is cleanup code
                     try:
@@ -63,10 +67,10 @@ class TestReraise(unittest.TestCase):
                     except ValueError:
                         # it's okay, we've handled it...
                         x = 7
-        except FooError, err:
+        except FooError as err:
             # This time we should get our original FooError
             self.assertEqual(str(err), "foo")
-        except ValueError, err:
+        except ValueError as err:
             self.assert_(False)
         else:
             self.assert_(False)
@@ -76,14 +80,14 @@ class TestReraise(unittest.TestCase):
         try:                            # this is outer code
             try:
                 raiser(FooError("foo"))
-            except FooError, err:
+            except FooError as err:
                 with exc.reraise():
                     # This is cleanup code -- whoops, uncaught exception!
                     x = int("7a")
-        except FooError, err:
+        except FooError as err:
             # reraise shouldn't discard cleanup exception
             self.assert_(False)
-        except ValueError, err:
+        except ValueError as err:
             self.assert_(True)
         else:
             self.assert_(False)
@@ -94,11 +98,11 @@ class TestReraise(unittest.TestCase):
         try:                            # this is outer code
             with exc.reraise():
                 pass
-        except FooError, err:
+        except FooError as err:
             self.assert_(False)
-        except ValueError, err:
+        except ValueError as err:
             self.assert_(False)
-        except TypeError, err:
+        except TypeError as err:
             self.assert_(False)
         else:
             self.assert_(True)
@@ -228,7 +232,7 @@ class TestRetry(unittest.TestCase):
     def test_ret_good(self):
         iterator = iter(("bar", "bar", "foo"))
         betweens = set()
-        self.assertEqual(exc.retry_func(reject_bar, iterator.next,
+        self.assertEqual(exc.retry_func(reject_bar, partial(next, iterator),
                                         exc=BarError, times=3, between=betweens.add),
                          "foo")
         self.assertEqual(betweens, set((1, 2)))
@@ -237,7 +241,7 @@ class TestRetry(unittest.TestCase):
         iterator = iter(("bar", "bar", "bar"))
         betweens = set()
         try:
-            exc.retry_func(reject_bar, iterator.next,
+            exc.retry_func(reject_bar, partial(next, iterator),
                            exc=BarError, times=3, between=betweens.add)
         except BarError:
             self.assertEqual(betweens, set((1, 2)))
@@ -255,5 +259,34 @@ class TestRetry(unittest.TestCase):
         self.assertEqual(func.count, 2)
         self.assertEqual(func.betweens, set((1,)))
 
+class TestExchain(unittest.TestCase):
+    def setUp(self):
+        self.flat = [RuntimeError('bad'), ValueError('worse'), OSError('awful')]
+        # use copies so as not to confuse the issue by modifying self.flat
+        self.chained = copy.copy(self.flat[0])
+        self.chained.__context__ = copy.copy(self.flat[1])
+        self.chained.__context__.__context__ = copy.copy(self.flat[2])
+    
+    def test_exchain(self):
+        # compare classes and string values because a copy of an exception
+        # subclass instance doesn't normally compare equal to the original
+        chain = list(exc.exchain(self.chained))
+        self.assertEqual([x.__class__ for x in chain], [x.__class__ for x in self.flat])
+        self.assertEqual([str(x)      for x in chain], [str(x)      for x in self.flat])
+        chain = list(exc.rexchain(self.chained))
+        self.assertEqual([x.__class__ for x in chain], [x.__class__ for x in reversed(self.flat)])
+        self.assertEqual([str(x)      for x in chain], [str(x)      for x in reversed(self.flat)])
+
+    def test_caused_by(self):
+        # None of the exceptions in self.chained is literally an Exception,
+        # but all are instances of Exception subclasses. Stop at the first
+        # matching instance.
+        self.assertEqual(exc.caused_by(self.chained, Exception).__class__, RuntimeError)
+        self.assertEqual(exc.caused_by(self.chained, RuntimeError).__class__, RuntimeError)
+        self.assertEqual(exc.caused_by(self.chained, ValueError).__class__, ValueError)
+        self.assertEqual(exc.caused_by(self.chained, OSError).__class__, OSError)
+        self.assertEqual(exc.caused_by(self.chained, FooError), None)
+        self.assertEqual(exc.caused_by(self.chained, (FooError, OSError)).__class__, OSError)
+
 if __name__ == "__main__":
     unittest.main()
